# functions for the simulation study using our R-package "CDVineMLE"

# load package "CDRVineMLE"
#library("CDVineMLE")
#library("mvtnorm")


#############################################
# tau_frank
# Input:
# tau		Kendall's tau
# Output:
# theta		computed parameter for the Frank-copula
#############################################

tau_frank=function(tau)
{
# Build a table of tau's for Frank copula
	# The parameter has to be non-zero

	options(digits=5)
	delta1=seq(0.01,5,0.01)
	delta2=seq(5.1,20,0.1)
	delta3=seq(21,50,1)
	delta4=seq(-5,-0.01,0.01)
	delta5=seq(-20,-5.1,0.1)
	delta6=seq(-50,-21,1)
	delta_v=c(delta1,delta2,delta3)
	delta_v2=c(delta6,delta5,delta4)
	delta_v3=c(delta_v2,delta_v)

	n=length(delta_v)
	n2=length(delta_v2)
	tau_v=rep(0,n)
	tau_v2=rep(0,n2)
	f=function(x) {x/(exp(x)-1)}
	for (i in 1:n)
	{
	tau_v[i]=1-4/delta_v[i]+4/delta_v[i]^2*integrate(f,lower=0, upper=delta_v[i])$value
	}
	for (i in 1:n2)
	{
	tau_v2[i]=1-4/delta_v2[i]-4/delta_v2[i]^2*integrate(f,lower=delta_v2[i], upper=0)$value
	}

	tau_v=c(tau_v2,tau_v)

	# Compare the estimated tau with the values of the table

	i=0
	for (j in 1:(n+n2))
	{
	if((tau_v[j]<tau | tau_v[j]==tau) & tau<tau_v[j+1])
	{
	i=j
	break 
	}
	}

	theta=delta_v3[i]
	return(theta)
}


#############################################
# tau_joe
# Input:
# tau		Kendall's tau
# Output:
# theta		computed parameter for the Joe-copula
#############################################

tau_joe=function(tau)
{
# Build a table of tau's for Joe copula
	# The parameter has to be greater or even one
	options(digits=5)
	delta1=seq(1,1.99,0.01)
	delta2=seq(2.01,5,0.01)
	delta3=seq(5.1,10,0.1)
	delta4=seq(11,50,1)

	delta_v=c(delta1,delta2,delta3,delta4)

	n=length(delta_v)
	tau_v=rep(0,n)
	euler=0.57721
	for (i in 1:n)
	{
	tau_v[i]=1+((-2+2*euler+2*log(2)+digamma(1/delta_v[i])+digamma(1/2*(2+delta_v[i])/delta_v[i])+delta_v[i])/(-2+delta_v[i]))
	}

	# Compare the estimated tau with the values of the table

	i=0
	for (j in 1:n)
	{
	if((tau_v[j]<tau | tau_v[j]==tau) & tau<tau_v[j+1])
	{
	i=j
	break 
	}
	}
	theta=delta_v[i]
return(theta)
}


#########################################################
# delta_bb7
# Input:
# theta		parameter theta of the BB7-copula
# tau		Kendall's tau
# Output:
# delta		second parameter of the BB7-copula
########################################################

delta_bb7=function(theta,tau)
{
delta1=seq(0.01,1.99,0.01)
delta2=seq(2.01,5,0.1)
delta3=seq(5.1,10,0.1)
delta4=seq(11,50,1)
		
delta_v=c(delta1,delta2,delta3,delta4)
n2=length(delta_v)
tau_m = rep(0,n2)

for(j in 1:n2)
{
	tau_m[j]=1-2/(delta_v[j]*(2-theta))+4/(theta^2*delta_v[j])*gamma(delta_v[j]+2)*gamma((2-2*theta)/(theta)+1)/gamma(delta_v[j]+3+(2-2*theta)/(theta))
}
i=0
for (j in 1:n2)
{
	if((tau_m[j]<tau | tau_m[j]==tau) & tau<tau_m[j+1])
	{
	i=j
	break 
	}
}
return(delta_v[i])
}


#################################################################
# tail_dependence	(macht keinen Sinn)
# Input:
# lambda	tail-dependence behavior ("S"=symetric, "NS"=non-symetric)
# tau		Kendall's tau
# copula	copula ("BB1", "BB7")
# Output:
# param		calculated parameters
################################################################

tail_dependence<-function(lambda,tau,copula)
{
	param=re(0,2)

	if(copula=="BB1")
	{
		if(tau>=0.8)
		{
		func<-function(delta,tau){2-2^(1/delta)-2^(-(1-tau)/(2-2*delta*(1-tau)))}
		param[2] <- uniroot(func,c(-5,20),tau=tau)$root
		}
		else
		{
		stop("For this combination of tau and lambda there is no solution in the region [1,infinity]")
		}
		param[1]=2/(param[2]*(1-tau))-2
	}
	else if(copula=="BB7")
	{
	}

return(param)
}


##################################################################
# parameter
# Input:
# copula	copula ("N","t3","t5","t10","C","G","F","J","BB1",BB7")
# tau		Kendall's tau of the copula
# lambdaU	upper tail dependence (for BB1 and BB7)
# Output:
# param		computed parameter(s)
#################################################################

parameter<-function(copula,tau,lambdaU=0)
{
	param=rep(0,2)
	if(copula=="N" | copula=="t3" | copula=="t5" | copula=="t10")
	{
		param[1]=sin(tau*pi/2)
		if(copula=="t3"){param[2]=3}
		else if(copula=="t5"){param[2]=5}
		else if(copula=="t10"){param[2]=10}
	}
	else if(copula=="C")
	{
		param[1]=max(0,2*tau/(1-tau))
	}
	else if(copula=="G")
	{
		param[1]=max(1,1/(1-tau))
	}
	else if(copula=="F")
	{
		param[1]=tau_frank(tau)
	}
	else if(copula=="J")
	{
		param[1]=tau_joe(tau)
	}
	else if(copula=="BB1")
	{
		param[2]=1/(log(2-lambdaU,2))
		param[1]=2/((1-tau)*param[2])-2
	}
	else if(copula=="BB7")
	{
		param[1]=1/(log(2-lambdaU,2))
		param[2]=delta_bb7(param[1],tau)
	}

return(param)
}


########################################################
# ftau
# Input:
# parma		parameter of the copula
# copula	copula family
# Output:
# tau		Kendall's tau
#######################################################

ftau<-function(param,copula)
{
	if(copula=="N" | copula=="t3" | copula=="t5" | copula=="t10")
	{
		tau=2/pi*asin(param)
	}
	else if(copula=="C")
	{
		tau=param/(param+2)
	}
	else if(copula=="G")
	{
		tau=1-1/param
	}
	else if(copula=="F")
	{
		f=function(x) {x/(exp(x)-1)}
		tau=1-4/param+4/param^2*integrate(f,lower=0, upper=param)$value
	}
	else if(copula=="J")
	{
		euler=0.57721
		tau=1+((-2+2*euler+2*log(2)+digamma(1/param)+digamma(1/2*(2+param)/param)+param)/(-2+param))
	}
	else if(copula=="BB1")
	{
		theta=param[1]
		delta=param[2]
		tau=1-2/(delta*(theta+2))
	}
	else if(copula=="BB7")
	{
		theta=param[1]
		delta=param[2]
		tau=1-2/(delta*(2-theta))+4/(theta^2*delta)*gamma(delta+2)*gamma((2-2*theta)/(theta)+1)/gamma(delta+3+(2-2*theta)/(theta))

	}

return(tau)
}


##########################################################
# copFam
# Input:
# copula	copula in the form of one of N,t3,t5,t10,C,G,F,J,BB1,BB7
# Output:
# family	copula in the form of the notation of the package "VineMLE"
##########################################################

copFam<-function(copula)
{
	family=0
	if(copula=="N"){family=1}
	else if(copula=="t3" | copula=="t5" | copula=="t10"){family=2}
	else if(copula=="C"){family=3}
	else if(copula=="G"){family=4}
	else if(copula=="F"){family=5}
	else if(copula=="J"){family=6}
	else if(copula=="BB1"){family=7}
	else if(copula=="BB7"){family=8}
	
return(family)
}


###########################################################
# simulate
# Input:
# copulas	vector of copulas ("N","t3","t5","t10","C","G","F","J","BB1",BB7")
# n		number of simulations
# taus		vector of Kendall's tau
# lambdaUs	vector of upper tail dependence (same length as taus)
# Output:
# daten		matrix of simulated data
###########################################################

simulate<-function(copulas,n,taus,lambdaUs)
{
	d1=length(copulas)
	d=(1+sqrt(1+8*d1))/2
	param=matrix(0,nrow=d1,ncol=2)
	daten=matrix(0,nrow=n,ncol=d)
	family=rep(0,d1)

	for(i in 1:d1)
	{
		param[i,]=parameter(copulas[i],taus[i],lambdaUs[i])
		family[i]=copFam(copulas[i])
	}

	theta=param[,1]
	nu=param[,2]

	daten=SimulateCopulaVine(n,d,family,theta,nu,1)

return(daten)
}


#######################################################
# testMLE
# Input:
# copulas	vector of copulas ("N","t3","t5","t10","C","G","F","J","BB1",BB7")
# n		number of simulations
# taus		vector of Kendall's tau
# lambdaUs	vector of upper tail dependence (same length as taus)
# r		number of replications
# maxiter	max. number of iterations (for the function "VineMLE")
# Output:
# paramMLE	matrix (r*d) of estimated parameters
#######################################################

testMLE<-function(copulas,n,taus,lambdaUs,r=100,maxiter=200)
{
	d1=length(copulas)
	d=(1+sqrt(1+8+d1))/2
	daten=matrix(0,nrow=n,ncol=d)
	family=rep(0,d1)
	tt=0

	for(i in 1:d1)
	{
		family[i]=copFam(copulas[i])
	}

	tt=sum(family==2)+sum(family==7)+sum(family==8)
	paramMLE=matrix(0,nrow=r,ncol=d1+tt)

	for(i in 1:r)
	{
		daten=simulate(copulas,n,taus,lambdaUs)
		paramMLE[i,]=VineMLE(daten,family,maxiter,type=1)$par
	}

return(paramMLE)
}


#################################################################
# Serien testen
# Input:
# copulas	vector of copulas to test
# n		Number of Iterations
# lambdaUs	vector of upper tail dependence behavior
# maxiter	Max. number of iterations (for the function "VineMLE")
# Output:
# serie		list of 4 matrix with the estimated parameters
#################################################################

serie_testen<-function(copulas,n,lambdaUs=matrix(0,nrow=4,ncol=6),maxiter=200)
{
	# Festsetzen der Parameter
	dd=length(copulas)		#dd=6 (fest)
	r=100
	family=rep(0,dd)

	for(i in 1:dd)
	{
		family[i]=copFam(copulas[i])
	}
	tt=sum(family==2)+sum(family==7)+sum(family==8)

	tau_h=0.8	# high Kendall's tau 
	tau_l=0.2	# low Kendall's tau
	tau=matrix(0,nrow=4,ncol=6)
	tau[1,]=c(tau_h,tau_h,tau_h,tau_h,tau_h,tau_h)
	tau[2,]=c(tau_l,tau_l,tau_l,tau_l,tau_l,tau_l)
	tau[3,]=c(tau_h,tau_h,tau_h,tau_l,tau_l,tau_l)
	tau[4,]=c(tau_l,tau_l,tau_l,tau_h,tau_h,tau_h)

	serie=list(array(rep(0,r*(dd+tt)),c(r,(dd+tt))), array(rep(0,r*(dd+tt)),c(r,(dd+tt))), array(rep(0,r*(dd+tt)),c(r,(dd+tt))), array(rep(0,r*(dd+tt)),c(r,(dd+tt))))

	for(i in 1:4)
	{
		serie[[i]]=testMLE(copulas,n,tau[i,],lambdaUs[i,],r=100,maxiter)
	}
	#mclapply(i=1:4,FUN=testMLE,copulas,n,tau,lambdaUs,r=100,maxiter=200)
	
return(serie)
}



#################################################################
# vuong.clarke.sim						#
#								#
# Input:							#
# xy		2-dim matrix of data				#
# tau		Kendall's tau					#
# lambdaU	upper tail dependence for BB1 and BB7 copulas	#
# cop.model1	character, copula on the left side		#
# cop.model2	vector of character, copula on the righ side	#
# Output:							#
# sim.result	Simulation of the models			#
#################################################################

vuong.clarke.sim<-function(xy,tau,lambdaU,cop.model1,cop.model2)
{
  cop.model2<-as.list(cop.model2)
  names(cop.model2)<-cop.model2

  #-----------------------------------------------------

  # begin of function code
  n<-dim(xy)[1]
  xy.rank<-apply(xy,2,rank)/(n+1)

  loglik.model1<-my.loglik(xy.rank,cop.model1,tau,lambdaU)
  loglik.model2<-data.frame(sapply(cop.model2,function(x){my.loglik(cop.name=x,xy.rank,tau,lambdaU)},USE.NAMES=TRUE))
  vuong.result.temp<-list()
  clarke.result.temp<-list()

  for(i in 1:length(loglik.model2))
  {
    p1<-1
    p2<-1
  
    model1.name<-cop.model1
    
    if(model1.name==names(loglik.model2)[i])
    {
      test.result<-data.frame(NA,NA,NA,NA)
      names(test.result)<-c("model","nu","p.value","kurtosis")
      vuong.result.temp[[i]]<-test.result
      names(vuong.result.temp)[i]<-names(loglik.model2)[i]
      clarke.result.temp[[i]]<-test.result
      names(clarke.result.temp)[i]<-names(loglik.model2)[i]
    }
    else if(model1.name!=names(loglik.model2)[i])
    {
      #print(paste("model 1 is ",model1.name,sep=""))
      if(is.element(model1.name,c("t3","t5","t10","BB1","BB7"))){ p1<-2}
  
      model2.name<-names(loglik.model2)[i]
      #print(paste("model 2 is ",model2.name,sep=""))
      if(is.element(model2.name,c("t3","t5","t10","BB1","BB7"))){p2<-2}
  
      #print(paste("p1=",p1,sep=""))
      #print(paste("p2=",p2,sep=""))
      vuong.result.temp[[i]]<-vuong.test(loglik.model1,loglik.model2[,i],p1=p1,p2=p2,print.result=FALSE)
      names(vuong.result.temp)[i]<-model2.name
      clarke.result.temp[[i]]<-clarke.test(loglik.model1,loglik.model2[,i],p1=p1,p2=p2,print.result=FALSE)
      names(clarke.result.temp)[i]<-model2.name
    }
  }
  sim.result<-list(vuong.result.temp,clarke.result.temp)
  names(sim.result)<-c("vuong","clarke")
  return(sim.result)
}


#########################################
# choose.cop				#
#					#
# Input:				#
# cop.name	copula family		#
# Output:				#
# cop.temp	copula class		#
#########################################

choose.cop<-function(cop.name)
{    
  if(cop.name=="N"){cop.temp<-normalCopula(0.5,dim=2)}
  if(cop.name=="t3"){cop.temp<-tCopula(0.5,dim=2,dispstr="un",df.fixed=FALSE,df=3)}
  if(cop.name=="t5"){cop.temp<-tCopula(0.5,dim=2,dispstr="un",df.fixed=FALSE,df=5)}
  if(cop.name=="t10"){cop.temp<-tCopula(0.5,dim=2,dispstr="un",df.fixed=FALSE,df=10)}
  if(cop.name=="C"){cop.temp<-claytonCopula(4,dim=2)}
  if(cop.name=="G"){cop.temp<-gumbelCopula(4,dim=2)}
  if(cop.name=="F"){cop.temp<-frankCopula(4,dim=2)}
  if(cop.name=="P"){cop.temp<-plackettCopula(2)}
  return(cop.temp)
}


#########################################
# my.loglik				#
#					#
# Input:				#
# xy.rank	Rank matrix		#
# cop.name	copula family		#
# tau		Kendall's tau		#
# lambdaU	upper tail dependence	#
# Output:				#
# loglik.temp	Log likelihood		#
#########################################



my.loglik<-function(xy.rank,cop.name,tau,lambdaU=0)
{
  if(is.element(cop.name,c("N","C","G","F","P")))
  {
    cop.temp<-choose.cop(cop.name)
    theta.temp<-parameter(cop.name,tau,lambdaU)[1]
    loglik.temp<-apply(xy.rank,1,function(x){loglikCopula(theta.temp,x,cop.temp)})
  }
  if(cop.name=="t3")
  {
    cop.temp<-choose.cop(cop.name)
    theta.temp<-parameter(cop.name,tau,lambdaU)[1]
    loglik.temp<-apply(xy.rank,1,function(x){loglikCopula(c(theta.temp,3),x,cop.temp)})
  }
  if(cop.name=="t5")
  {
    cop.temp<-choose.cop(cop.name)
    theta.temp<-parameter(cop.name,tau,lambdaU)[1]
    loglik.temp<-apply(xy.rank,1,function(x){loglikCopula(c(theta.temp,5),x,cop.temp)})
  }
  if(cop.name=="t10")
  {
    cop.temp<-choose.cop(cop.name)
    theta.temp<-parameter(cop.name,tau,lambdaU)[1]
    loglik.temp<-apply(xy.rank,1,function(x){loglikCopula(c(theta.temp,10),x,cop.temp)})
  }
  if(cop.name=="BB1")
  {
    theta.delta.temp<-parameter(cop.name,tau,lambdaU)
    loglik.temp<-apply(xy.rank,1,function(x){llbb1(theta.delta.temp,x[1],x[2])})
  }
  if(cop.name=="BB7")
  {
    theta.delta.temp<-parameter(cop.name,tau,lambdaU)
    loglik.temp<-apply(xy.rank,1,function(x){llbb7(theta.delta.temp,x[1],x[2])})
  }
  return(loglik.temp)
}


#################################################################
# vuong.test							#
#								#
# Input:							#
# loglik.model1	vector of log likelihood of the first model	#
# loglik.model1	vector of log likelihood of the second model	#
# alpha		alpha-level for p-value				#
# p1		number of parameters (model 1)			#
# p2		number of parameters (model 2)			#
# Output:							#
# Vuong test							#
#################################################################


vuong.test<-function(loglik.model1,loglik.model2,alpha=0.05,p1=0,p2=0,print.result=TRUE)
{
  # load required packages - timeDate etc.
  if(!is.element(c("package:timeDate"),search())){library("timeDate")}
  
  #print(p1)
  #print(p2)
  
  H0<-"H0: model (1) is eqivalent to model (2)"
  if(print.result)print(H0)
  n<-length(loglik.model1)
  m.i<-loglik.model1-loglik.model2-(p1-p2)*log(n)/(2*n)
  kurt.ratios<-kurtosis(loglik.model1-loglik.model2)
  nu<-(sqrt(n)*mean(m.i))/(sqrt((n-1)/n*var(m.i)))
  #print(paste("nu =",formatC(nu,3,format="g")))
  if(abs(nu)<qnorm(1-alpha/2))
  {
    decision<-"Decision: non of the models is favoured"
    result<-0
  }
  if(nu >=qnorm(1-alpha/2) )
  {
    decision<-"Decision: favour model 1"
    result<-1
  }
  if(nu <= -qnorm(1-alpha/2) )
  {
    decision<-"Decision: favour model 2"
    result<-2
  }
  if(print.result)print(decision)
  pvalue<-2*pnorm(-abs(nu))
  result<-data.frame(result,round(nu,digits=3),round(pvalue,digits=3),round(kurt.ratios,digits=3))
  names(result)<-c("model","nu","p.value","kurtosis")
  if(print.result)print(result)
  if(print.result)print("#----------------------------------------------")
  return(result)
  rm(n,m.i,kurt.ratios,nu,pvalue,result,decision) 
}


#################################################################
# clarke.test							#
#								#
# Input:							#
# loglik.model1	vector of log likelihood of the first model	#
# loglik.model1	vector of log likelihood of the second model	#
# alpha		alpha-level for p-value				#
# p1		number of parameters (model 1)			#
# p2		number of parameters (model 2)			#
# Output:							#
# Clarke test							#
#################################################################

clarke.test<-function(loglik.model1,loglik.model2,alpha=0.05,p1=0,p2=0,print.result=TRUE)
{ 
  # load required packages - timeDate etc.
  if(!is.element(c("package:timeDate"),search())){library("timeDate")}
  
  #print(p1)
  #print(p2)
  
  H0<-"H0: model (1) is eqivalent to model (2)"
  if(print.result)print(H0)
  n<-length(loglik.model1)
  m.i<-loglik.model1-loglik.model2-(p1-p2)*log(n)/(2*n)
  kurt.ratios<-kurtosis(loglik.model1-loglik.model2)
  B<-sum(m.i > 0)
  cAlpha<-(n/2-qbinom(p=alpha/2,n,prob=1/2))# das ist richtig
  if(abs(B-n/2)<cAlpha)
  {
    decision<-"Decision: Non of the models is favoured"
    result<-0
  }
  if((B-n/2)>=cAlpha)
  {
    decision<-"Decision: Favour model 1"
    result<-1
  }
  if((B-n/2)<=((-1)*cAlpha))
  {
    decision<-"Desicion: Favour model 2"
    result<-2
  }
  if(print.result)print(decision)
  pvalue<-min(2*pbinom(B,size=n,prob=0.5),2*(1-pbinom(B-1,size=n,prob=0.5)))
  result<-data.frame(result,round(B,digits=3),round(pvalue,digits=3),round(kurt.ratios,digits=3))
  names(result)<-c("model","B","p.value","kurtosis")
  if(print.result)print(result)
  if(print.result)print("#----------------------------------------------")
  return(result)
  rm(n,m.i,kurt.ratios,B,cAlpha,result,decision,pvalue) 
}


#################################
# Score Funktion		#
#				#
# Input:			#
# dat		data matrix	#
# Output:			#
# score				#
#################################

score<-function(dat)
{
  yes<-sum(dat==1,na.rm=TRUE)  # zhle wieviel mal kommt "yes" vor
  no<-sum(dat==2,na.rm=TRUE)*(-1)  # zhle wieviel mal kommt "no" vor
  return(yes+no)
}


#################################################
# Goodness-of-fit test with Vuong and Clarke	#
#						#
# Input:					#
# xy		data matrix (2-dim)		#
# cop.model2	vector of copula families	#
# tau.fit	empirical Kendall's tau		#
# lambdaU	upper tail dependence		#
# Output:					#
# score.gof	Goodness-of-fit score		#
#################################################


gofVC<-function(xy,cop.model2,tau.fit,lambdaU)
{
score.gof<-matrix(rep(NA,2*length(cop.model2)),nrow=2)
dimnames(score.gof)<-list(c("vuong","clarke"),cop.model2)

for(i in 1:length(cop.model2))
{
  temp<-vuong.clarke.sim(xy,tau.fit,lambdaU,cop.model2[i],cop.model2)
  temp.vuong<-c()
  temp.clarke<-c()
  for(j in 1:length(temp$vuong))
  {
     temp.vuong<-cbind(temp.vuong,t(temp$vuong[[j]]))
     dimnames(temp.vuong)[[2]][j]<-names(temp$vuong)[j]
     temp.clarke<-cbind(temp.clarke,t(temp$clarke[[j]]))
     dimnames(temp.clarke)[[2]][j]<-names(temp$clarke)[j]
  }
  score.temp<-rbind(score(temp.vuong[1,]),score(temp.clarke[1,]))
  score.gof[,i]<-score.temp
}

return(score.gof)
}



#################################################################
# bind.vuong.clarke						#
#								#
# Input:							#
# vuong		vector with the results of the Vuong test	#
# clarke	vector with the results of the Clarke test	#
# Output:							#
# vuong.clarke	matrix of the tests				#
#################################################################

bind.vuong.clarke<-function(vuong,clarke)
{
  clarke<-lapply(clarke,function(x){
    yes<-round(sum(ifelse(x$model==1,1,0))/m*100,digits=2)
    non<-round(sum(ifelse(x$model==0,1,0))/m*100,digits=2)
    no<-round(sum(ifelse(x$model==2,1,0))/m*100,digits=2)
    kurtosis<-round(mean(x$kurtosis),digits=2)
    result<-t(data.frame(yes,non,no,kurtosis))
    dimnames(result)[[2]]<-"clarke"
    return(result)
  })
  
  vuong<-lapply(vuong,function(x){
    yes<-round(sum(ifelse(x$model==1,1,0))/m*100,digits=2)
    non<-round(sum(ifelse(x$model==0,1,0))/m*100,digits=2)
    no<-round(sum(ifelse(x$model==2,1,0))/m*100,digits=2)
    kurtosis<-round(mean(x$kurtosis),digits=2)
    result<-t(data.frame(yes,non,no,kurtosis))
    dimnames(result)[[2]]<-"vuong"
    return(result)
  })

  vuong.clarke<-c()
  vuong.clarke$normal<-cbind(vuong$normal,clarke$normal)
  vuong.clarke$t<-cbind(vuong$t,clarke$t)
  vuong.clarke$clayton<-cbind(vuong$clayton,clarke$clayton)
  vuong.clarke$gumbel<-cbind(vuong$gumbel,clarke$gumbel)
  vuong.clarke$frank<-cbind(vuong$frank,clarke$frank)
  vuong.clarke$plackett<-cbind(vuong$plackett,clarke$plackett)
  vuong.clarke$bb1<-cbind(vuong$bb1,clarke$bb1)
  vuong.clarke$bb7<-cbind(vuong$bb7,clarke$bb7)

  vuong.clarke<-cbind(vuong.clarke$normal,vuong.clarke$t,vuong.clarke$clayton,vuong.clarke$gumbel,
    vuong.clarke$frank,vuong.clarke$plackett,vuong.clarke$bb1,vuong.clarke$bb7)

  vuong.clarke<-rbind(c("normal","normal","t","t","clayton","clayton","gumbel","gumbel",
    "frank","frank","plackett","plackett","bb1","bb1","bb7","bb7"),dimnames(vuong.clarke)[[2]],vuong.clarke)
  
  dimnames(vuong.clarke)[[1]][1:2]<-c("copula","test")
  vuong.clarke<-cbind(dimnames(vuong.clarke)[[1]],vuong.clarke)
  dimnames(vuong.clarke)<-NULL
  
  return(vuong.clarke)
}


#################################################
# LogLikelihood					#
# Input:					#
# data		data vector			#
# cop		copula family vector		#
# par		first parameter vector		#
# nu		second parameter vector		#
# type		vine type			#
# Output:					#
# loglik	log-likelihood			#
#################################################

LogLikelihood<-function(cop,data,par,nu=rep(0,length(data)),type)
{
	loglik=0
	d=length(data)
	ll <- rep(0.,d*(d-1)/2)
	if(type==1) vv <- rep(0,T*(((d-1)*d)/2-1))
	else vv <- rep(0,T*(d-1)*(d-2))
	out <- .C("VineLogLikm",
              as.integer(1),
              as.integer(d),
              as.integer(type),
              as.integer(cop),
              as.double(c(par,nu)),
              as.double(data),
              as.double(0),
              as.double(ll),
              as.double(vv),
              PACKAGE='CDVineMLE')
	loglik <- -out[[7]]
	if (loglik %in% c(NA,NaN,-Inf)) loglik <- -1e10

return(loglik)
}
